<?php

namespace Drupal\menu_block_settings_override\Form;

use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\ConfigFormBase;

/**
 * Provides the menu block settings override administration form.
 *
 * @internal
 */
class MenuBlockSettingsOverrideAdmin extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The block manager.
   *
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  protected $blockManager;

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'menu_block_settings_override.settings';

  /**
   * Creates a CommentAdminOverview form.
   *
   * @param \Drupal\Core\Block\BlockManagerInterface $block_manager
   *   The block manager service.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity manager service.
   */
  public function __construct(BlockManagerInterface $block_manager, EntityTypeManagerInterface $entity_type_manager) {
    $this->blockManager = $block_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.block'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'menu_block_settings_override_admin';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * Form constructor for the menu block settings override administration form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param string $type
   *   The type of the overview form ('approval' or 'new').
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $type = 'new') {
    // Load all menu blocks.
    $blocks_options = [];
    $config = $this->config(static::SETTINGS);

    $menu_blocks = array_filter($this->blockManager->getDefinitions(), function ($d) {
      return $d['id'] == 'menu_block';
    });

    if (!empty($menu_blocks)) {
      foreach ($menu_blocks as $id => $block) {
        $blocks_options[$id] = $block['admin_label'];
      }
      $form['blocks_enabled'] = [
        '#title' => $this->t('Enable per-page Menu block settings overrides for the following blocks:'),
        '#type' => 'checkboxes',
        '#options' => $blocks_options,
        '#default_value' => $config->get('blocks_enabled', []),
      ];
    }

    $form['hidden_settings'] = [
      '#title' => $this->t('Hide the following settings on the per-page settings:'),
      '#description' => $this->t('Any settings in this list will not be overridden on a per-page basis and will read from the global block-level settings instead'),
      '#type' => 'checkboxes',
      '#options' => [
        'level' => $this->t('Initial visibility level'),
        'depth' => $this->t('Number of levels to display'),
        'expand_all_items' => $this->t('Expand all menu items'),
        'follow' => $this->t('Make the initial visibility level follow the active menu item'),
        'follow_parent' => $this->t('Initial visibility level will be'),
        'expand' => $this->t('Expand all menu links'),
        'parent' => $this->t('Fixed parent item'),
        'suggestion' => $this->t('Theme suggestion'),
      ],
      '#default_value' => $config->get('hidden_settings', []),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->configFactory->getEditable(static::SETTINGS)
      ->set('blocks_enabled', $form_state->getValue('blocks_enabled'))
      ->set('hidden_settings', $form_state->getValue('hidden_settings'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
