<?php

namespace Drupal\menu_block_settings_override\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\Field\FieldItemBase;

/**
 * Defines a menu_block_settings_override field type which stores overridden menu block settings.
 *
 * @FieldType(
 *   id = "menu_block_settings_override",
 *   label = @Translation("Menu block settings override"),
 *   description = @Translation("This field stores overridden menu block settings on a node."),
 *   list_class = "\Drupal\menu_block_settings_override\Plugin\Field\MenuBlockSettingsOverrideFieldItemList",
 *   cardinality = \Drupal\Core\Field\FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED,
 *   default_widget = "menu_block_settings_override",
 * )
 */
class MenuBlockSettingsOverrideItem extends FieldItemBase {

  /**
   * @var \Drupal\Core\Config\ConfigFactoryInterface|null
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['menu_block_id'] = DataDefinition::create('string')
      ->setLabel(t('Menu block ID'));

    $properties['level'] = DataDefinition::create('integer')
      ->setLabel(t('Starting level'));

    $properties['depth'] = DataDefinition::create('integer')
      ->setLabel(t('Maximum number of levels'));

    $properties['follow'] = DataDefinition::create('boolean')
      ->setLabel(t('The initial visibility level follows the active menu item'));

    $properties['follow_parent'] = DataDefinition::create('string')
      ->setLabel(t('Set initial visibility level to the active menu item, or its children'));

    $properties['expand'] = DataDefinition::create('boolean')
      ->setLabel(t('Expand all menu links'));

    $properties['parent'] = DataDefinition::create('string')
      ->setLabel(t('Parent menu link'));

    $properties['suggestion'] = DataDefinition::create('string')
      ->setLabel(t('Theme suggestion'));

    $properties['hidden'] = DataDefinition::create('boolean')
      ->setLabel(t('Whether the menu block is hidden on this entity'));

    $properties['status'] = DataDefinition::create('boolean')
      ->setLabel(t('Whether the menu block settings ought to be overridden on this entity'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'menu_block_id' => [
          'description' => 'Menu block ID',
          'type' => 'varchar',
          'length' => 255,
          'not null' => TRUE,
        ],
        'follow' => [
          'description' => 'The initial visibility level follows the active menu item',
          'type' => 'int',
          'not null' => TRUE,
        ],
        'follow_parent' => [
          'description' => 'Set initial visibility level to the active menu item, or its children',
          'type' => 'varchar',
          'length' => 255,
          'not null' => TRUE,
        ],
        'level' => [
          'description' => 'Starting level',
          'type' => 'int',
          'not null' => TRUE,
        ],
        'depth' => [
          'description' => 'Maximum number of levels',
          'type' => 'int',
          'not null' => TRUE,
        ],
        'expand' => [
          'description' => 'Expand all menu links',
          'type' => 'int',
          'not null' => TRUE,
        ],
        'parent' => [
          'description' => 'Parent menu link',
          'type' => 'varchar',
          'length' => 255,
          'not null' => TRUE,
        ],
        'suggestion' => [
          'description' => 'Theme suggestion',
          'type' => 'varchar',
          'length' => 255,
          'not null' => TRUE,
        ],
        'hidden' => [
          'description' => 'Whether the menu block is hidden on this entity',
          'type' => 'int',
          'not null' => TRUE,
        ],
        'status' => [
          'description' => 'Whether menu block settings are overridden on this entity',
          'type' => 'int',
          'not null' => TRUE,
        ],
      ],
      'primary key' => ['menu_block_id'],
      'indexes' => [],
      'foreign keys' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->getValue();
    return empty($value['status']) && empty($value['hidden']);
  }

  /**
   * @return \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected function getConfigFactory() {
    if (!$this->configFactory) {
      $this->configFactory = \Drupal::configFactory();
    }
    return $this->configFactory;
  }

  /**
   * @return array Full list of all menu_block fields that can be overridden by menu_block_settings_override.
   */
  public static function getOverridableFields() {
    return [
      'follow',
      'follow_parent',
      'level',
      'depth',
      'expand',
      'parent',
      'suggestion',
    ];
  }

  /**
   * @param string $field_name
   * @return bool Whether the value in the field should be taken into account when overriding menu block settings.
   */
  public function isApplicable($field_name) {
    if (!in_array($field_name, $this->getOverridableFields())) {
      return FALSE;
    }
    // TODO: inject once https://www.drupal.org/project/drupal/issues/2053415 lands.
    $config_factory = $this->getConfigFactory();
    $config = $config_factory->get('menu_block_settings_override.settings');
    return !in_array($field_name, array_values(array_filter($config->get('hidden_settings'))));
  }

}
