<?php

namespace Drupal\menu_block_settings_override\Plugin\Field\FieldWidget;

use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Element;
use Drupal\menu_block_settings_override\Plugin\Field\FieldType\MenuBlockSettingsOverrideItem;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a widget for the menu_block_settings_override field type.
 *
 * @FieldWidget(
 *   id = "menu_block_settings_override",
 *   label = @Translation("Menu block settings override"),
 *   multiple_values = TRUE,
 *   field_types = {
 *     "menu_block_settings_override"
 *   }
 * )
 */
class MenuBlockSettingsOverrideWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  protected $blockManager;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, ConfigFactoryInterface $config_factory, BlockManagerInterface $block_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->configFactory = $config_factory;
    $this->blockManager = $block_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('config.factory'),
      $container->get('plugin.manager.block')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('menu_block_settings_override.settings');
    $entity = $items->getEntity();
    $blocks_enabled = array_values(array_filter($config->get('blocks_enabled')));
    if (empty($blocks_enabled)) {
      return [];
    }
    $hidden = 0;
    $status = 0;
    foreach ($blocks_enabled as $block_id) {
      /** @var \Drupal\menu_block_settings_override\Plugin\Field\FieldType\MenuBlockSettingsOverrideItem|bool $value */
      $value = $items->getValueForMenuBlockId($block_id);

      if (!empty($value)) {
        $hidden = $value->get('hidden')->getCastedValue();
        $status = $value->get('status')->getCastedValue();
      }
      $definition = $this->blockManager->getDefinition($block_id);
      if (!$definition) {
        continue;
      }
      /**
       * @var MenuBlock $instance
       */
      $instance = $this->blockManager->createInstance($block_id);
      $label = $definition['admin_label'];
      $element['menu_block_settings_override_' . $block_id] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Menu block settings: %title', ['%title' => $label]),
        '#attributes' => ['class' => ['menu-block-settings-override-form']],
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
        '#weight' => 40,
        '#tree' => TRUE,
      ];

      $element['menu_block_settings_override_' . $block_id]['hidden'] = [
        '#type' => 'checkbox',
        '#title' => t('Hide Menu block %title on this page', ['%title' => $label]),
        '#default_value' => isset($hidden) ? $hidden : 0,
      ];
      $element['menu_block_settings_override_' . $block_id]['status'] = [
        '#type' => 'checkbox',
        '#title' => t('Override Menu block settings for %title on this page', ['%title' => $label]),
        '#default_value' => isset($status) ? $status : 0,
      ];

      $menu_block_form = $instance->blockForm([], $form_state);
      $overridable_fields = MenuBlockSettingsOverrideItem::getOverridableFields();
      foreach ($menu_block_form as $group_key => $group_form_element) {
        $children = Element::children($menu_block_form[$group_key]);

        foreach ($children as $key) {
          $hidden = array_values(array_filter($config->get('hidden_settings')));
          if (in_array($key, $hidden) || !in_array($key, $overridable_fields)) {
            $menu_block_form[$group_key][$key]['#access'] = FALSE;
          }
          elseif ($value) {
            $overridden_setting = $value->get($key);
            $menu_block_form[$group_key][$key]['#default_value'] = $overridden_setting->getCastedValue();
          }
        }

        $enabled = array_filter($children, function ($child) use ($menu_block_form, $group_key) {
          return $menu_block_form[$group_key][$child]['#access'] !== FALSE;
        });
        if (empty($enabled)) {
          $menu_block_form[$group_key]['#access'] = FALSE;
        }
      }
      $element['menu_block_settings_override_' . $block_id]['custom_settings'] = $menu_block_form;

      $element['menu_block_settings_override_' . $block_id]['custom_settings'] += [
        '#type' => 'fieldset',
        '#title' => t('Menu block custom settings: %title', ['%title' => $label]),
        '#states' => [
          'visible' => [
            ':input[name*="menu_block_settings_override_' . $block_id . '][status]"]' => ['checked' => TRUE],
          ],
        ],
      ];
    }

    // If the advanced settings tabs-set is available (normally rendered in the
    // second column on wide-resolutions), place the field as a details element
    // in this tab-set.
    if (isset($form['advanced'])) {
      // Get default value from the field.
      $field_default_values = $this->fieldDefinition->getDefaultValue($entity);

      // Override widget title to be helpful for end users.
      $element['#title'] = $this->t('Menu block settings override');

      $element += [
        '#type' => 'details',
        '#open' => FALSE,
        '#group' => 'advanced',
      ];
    }
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as $key => &$value) {
      $value['menu_block_id'] = str_replace('menu_block_settings_override_', '', $key);
      if (!empty($value['custom_settings'])) {
        foreach ($value['custom_settings'] as $name => $setting) {
          $values[$key][$name] = $setting;
        }
        unset($value['custom_settings']);
      }
    }
    $values = parent::massageFormValues($values, $form, $form_state);
    return array_values($values);
  }

}
